#include <errno.h>
#include <fcntl.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdio.h>
#include <unistd.h>
#include <string>
#include "Configuration.h"
#include "ConfigEssentials.h"
#include "Constants.h"
#include "NcPrintErr.h"
#include "SerializerJson.h"


const char * GetDefaultUart()
{
	int fd = open( DEFAULT_UART_RPI, O_RDWR | O_NOCTTY | O_SYNC);
	if (fd<0)
		return DEFAULT_UART_LINUX;
	close(fd);
	return DEFAULT_UART_RPI;
}

bool SetSpeed(const char * speed, Configuration * c)
{
	if (strncmp(speed, CONFIG_SPEED_57600, strlen(CONFIG_SPEED_57600))==0)
		c->setSpeed(B57600);
	else if (strncmp(speed, CONFIG_SPEED_115200, strlen(CONFIG_SPEED_115200))==0)
		c->setSpeed(B115200);
	else if (strncmp(speed, CONFIG_SPEED_230400, strlen(CONFIG_SPEED_230400))==0)
		c->setSpeed(B230400);
	else
		return false;
	return true;
}

const char * MapSpeed(int speed)
{
	switch (speed)
	{
	case B57600:
		return CONFIG_SPEED_57600;
	case B115200:
		return CONFIG_SPEED_115200;
	case B230400:
		return CONFIG_SPEED_230400;
	default:
		return "Unsupported speed";
	}
}

bool SetCtsSource(const char * s, Configuration * c)
{
	bool ok = ((strncmp(s, CONFIG_CTSSOURCE_IOCTL, strlen(CONFIG_CTSSOURCE_IOCTL))==0)
	        || (strncmp(s, CONFIG_CTSSOURCE_GPIO, strlen(CONFIG_CTSSOURCE_GPIO))==0));
	if (ok)
		c->setCtsSource(s);
	return ok;
}

void GetConfigDefault(Configuration * c)
{
	c->setRecvPort( DEFAULT_RECV_PORT );
	c->setSendPort( DEFAULT_SEND_PORT );
	c->setUart( GetDefaultUart() );
	c->setSpeed( DEFAULT_BAUDRATE );
	SetSerializer( CONFIG_SERIALIZER_JSON, c );
	c->setCtsTimeoutSecs( DEFAULT_WRITTEN_TIMEOUT_SECS );
	c->setCtsSource( DEFAULT_CTSSOURCE );
	c->setGpioPin( DEFAULT_RPI_GPIO_PIN );
	c->setDelimiter( DEFAULT_DELIMETER );
}

bool GetConfigEntry(const char * s, Configuration * c)
{
	const char * s1 = strchr(s, '=');
	int tokenLen = s1-s;
	if (strncmp(s, CONFIG_UART, tokenLen)==0)
	{
		const char * uart = s1+1;
		int uartLen = strlen(uart);
		if (uart[uartLen-1]=='\n')
			uartLen--;
		c->setUart(strndup(uart, uartLen));
	}
	else if (strncmp(s, CONFIG_SPEED, tokenLen)==0)
	{
		const char * speed = s1+1;
		if (!SetSpeed(speed, c))
			return PrintErr("Invalid speed %s", speed);
	}
	else if (strncmp(s, CONFIG_RECVPORT, tokenLen)==0)
	{
		c->setRecvPort(atoi(s1+1));
	}
	else if (strncmp(s, CONFIG_SENDPORT, tokenLen)==0)
	{
		c->setSendPort(atoi(s1+1));
	}
	else if (strncmp(s, CONFIG_CTSTIMEOUTSECS, tokenLen)==0)
	{
		c->setCtsTimeoutSecs(atoi(s1+1));
	}
	else if (strncmp(s, CONFIG_CTSSOURCE, tokenLen)==0)
	{
		const char * src = s1+1;
		int srcLen = strlen(src);
		if (src[srcLen-1]=='\n')
			srcLen--;
		if (!SetCtsSource(strndup(src, srcLen), c))
			return PrintErr("Invalid %s: %s", CONFIG_CTSSOURCE, src);
	}
	else if (strncmp(s, CONFIG_SERIALIZER, tokenLen)==0)
	{
		const char * ser = s1+1;
		if (!SetSerializer(ser, c))
			return PrintErr("Invalid %s: %s", CONFIG_SERIALIZER, ser);
	}
	else if (strncmp(s, CONFIG_GPIOPIN, tokenLen)==0)
	{
		c->setGpioPin(atoi(s1+1));
	}
	else if (strncmp(s, CONFIG_DELIMITER, tokenLen)==0)
	{
		const char * del = s1+1;
		int delLen = strlen(del);
		if (del[delLen-1]=='\n')
			delLen--;
		c->setDelimiter(strndup(del, delLen));
	}
	else
		return PrintErr("Invalid config: %s", s);
	return true;
}

bool GetConfigFromFile(char * file, Configuration * c)
{
	FILE* f = fopen( file, "r");
	if (f==NULL)
		return PrintErr("Failed to open config: %s errno=%s", file, PrintErrNo(errno));
	char buf[4096];
	char * s;
	while ((s = fgets(buf, sizeof(buf), f))!=NULL)
		if (!GetConfigEntry(s, c))
			return false;
	fclose( f );
	return true;
}

bool GetConfigFromArgs(const char * arg, Configuration * c)
{
	const char * p = arg;
	const char * p1;
	while ((p1 = strchr(p, ','))!=NULL)
	{
		const char * ptmp = strndup(p, p1-p);
		if (!GetConfigEntry(ptmp, c))
			return false;
		p = p1+1;
		free((void*)ptmp);
	}
	return GetConfigEntry(p, c);;
}

bool GetConfig(int argc, char** args, Configuration * c)
{
	switch (argc)
	{
		case 1:
			break;
		case 3:
			if (strcmp(args[1], "-c")==0)
				return GetConfigFromFile(args[2], c);
			else if (strcmp(args[1], "-C")==0)
			{
				GetConfigDefault(c);
				return GetConfigFromArgs(args[2], c);
			}
			return PrintErr("Invalid argument: %s", args[1]);
		default:
			return PrintErr("Incorrect number of arguments");
	}
	GetConfigDefault(c);
	return true;
}

void PrintConfig(Configuration * c)
{
	printf("%s=%s\n", CONFIG_UART, c->getUart());
	printf("%s=%s\n", CONFIG_SPEED, MapSpeed(c->getSpeed()));
	printf("%s=%d\n", CONFIG_RECVPORT, c->getRecvPort());
	printf("%s=%d\n", CONFIG_SENDPORT, c->getSendPort());
	printf("%s=%s\n", CONFIG_SERIALIZER, c->getSerializerName());
	printf("%s=%d\n", CONFIG_CTSTIMEOUTSECS, c->getCtsTimeoutSecs());
	printf("%s=%s\n", CONFIG_CTSSOURCE, c->getCtsSource());
	printf("%s=%d\n", CONFIG_GPIOPIN, c->getGpioPin());
	printf("%s=%s\n", CONFIG_DELIMITER, c->getDelimiter());
}
